from typing import Dict, Tuple, List

import requests
import json
import os
import time

from compLib.LogstashLogging import Logging

API_URL = os.getenv("API_URL", "http://localhost:5000/") + "api/"
API_URL_GET_DELIVERY = API_URL + "getDelivery"
API_URL_GET_MATERIAL = API_URL + "getMaterial"
API_URL_GET_GARBAGE = API_URL + "getGarbage"
API_URL_GET_LIST_CARGO = API_URL + "listCargo"
API_URL_GET_CARGO = API_URL + "getCargo/"

API_URL_GET_POS = API_URL + "getPos"
API_URL_GET_OP = API_URL + "getOp"
API_URL_GET_GOAL = API_URL + "getGoal"
API_URL_GET_ITEMS = API_URL + "getItems"
API_URL_GET_SCORES = API_URL + "getScores"
API_URL_GET_METEOROID = API_URL + "getMeteoroid"


class Seeding:
    """Class used for communicating with seeding api
    """

    @staticmethod
    def get_delivery() -> Tuple[Dict, int]:
        """Makes the /api/getDelivery call to the api.

        :return: Json Object and status code as returned by the api.
        :rtype: Tuple[Dict, int]
        """
        res = requests.get(API_URL_GET_DELIVERY)
        result = json.loads(res.content)
        Logging.get_logger().debug(f"Seeding.get_delivery = {result}, status code = {res.status_code}")
        return result, res.status_code

    @staticmethod
    def get_material() -> Tuple[Dict, int]:
        """Makes the /api/getMaterial call to the api.

        :return: Json Object and status code as returned by the api.
        :rtype: Tuple[Dict, int]
        """
        res = requests.get(API_URL_GET_MATERIAL)
        result = json.loads(res.content)
        Logging.get_logger().debug(f"Seeding.get_material = {result}, status code = {res.status_code}")
        return result, res.status_code

    @staticmethod
    def get_garbage() -> Tuple[Dict, int]:
        """Makes the /api/getGarbage call to the api.

        :return: Json Object and status code as returned by the api.
        :rtype: Tuple[Dict, int]
        """
        res = requests.get(API_URL_GET_GARBAGE)
        result = json.loads(res.content)
        Logging.get_logger().debug(f"Seeding.get_garbage {result}, status code = {res.status_code}")
        return result, res.status_code

    @staticmethod
    def list_cargo() -> Tuple[Dict, int]:
        """Makes the /api/listCargo call to the api.

        :return: Json Object and status code as returned by the api.
        :rtype: Tuple[Dict, int]
        """
        res = requests.get(API_URL_GET_LIST_CARGO)
        result = json.loads(res.content)
        Logging.get_logger().debug(f"Seeding.list_cargo {result}, status code = {res.status_code}")
        return result, res.status_code

    @staticmethod
    def getCargo(color: str) -> Tuple[Dict, int]:
        """Makes the /api/listCargo call to the api.

        :param color: Color parameter which specifies which cargo should be taken. (A string which is either "green", "red", "yellow" or "blue"
        :return: Json Object and status code as returned by the api.
        :rtype: Tuple[Dict, int]
        """
        res = requests.get(API_URL_GET_CARGO + color)
        result = json.loads(res.content)
        Logging.get_logger().debug(f"Seeding.get_cargo {result}, status code = {res.status_code}")
        return result, res.status_code


class Position:
    """Datastructure for holding a position
    """

    def __init__(self, x, y, degrees):
        self.x = x
        self.y = y
        self.degrees = degrees

    def __repr__(self):
        return f"Position(x={self.x}, y={self.y}, degrees={self.degrees})"

    def __str__(self):
        return f"Position(x={round(self.x, 5)}, y={round(self.y, 5)}, degrees={round(self.degrees, 5)})"


class DoubleElim:
    """Class used for communicating with double elimination api
    """

    @staticmethod
    def get_position() -> Tuple[Position, int]:
        """Get position of the robot

        :return: A Position object with robot position
        :rtype: Tuple[Position, int]
        """
        res = requests.get(API_URL_GET_POS)
        if res.status_code == 408:
            Logging.get_logger().error(f"DoubleElim.get_position timeout!")
            time.sleep(0.01)
            return DoubleElim.get_position()

        response = json.loads(res.content)
        Logging.get_logger().debug(f"DoubleElim.get_position = {response}, status code = {res.status_code}")
        return Position(response["x"], response["y"], response["degrees"]), res.status_code

    @staticmethod
    def get_opponent() -> Tuple[Position, int]:
        """Get position of the opponents robot

        :return: A Position object with opponents robot position
        :rtype: Tuple[Position, int]
        """
        res = requests.get(API_URL_GET_OP)
        if res.status_code == 408:
            Logging.get_logger().error(f"DoubleElim.get_opponent timeout!")
            time.sleep(0.01)
            return DoubleElim.get_opponent()

        response = json.loads(res.content)
        Logging.get_logger().debug(f"DoubleElim.get_opponent = x:{response}, status code = {res.status_code}")
        return Position(response["x"], response["y"], response["degrees"]), res.status_code

    @staticmethod
    def get_goal() -> Tuple[Position, int]:
        """Get position of the goal

        :return: A Position object with x and y coordinates of the goal, rotation is always -1
        :rtype: Tuple[Position, int]
        """
        res = requests.get(API_URL_GET_GOAL)
        if res.status_code == 408:
            Logging.get_logger().error(f"DoubleElim.get_goal timeout!")
            time.sleep(0.01)
            return DoubleElim.get_goal()

        response = json.loads(res.content)
        Logging.get_logger().debug(f"DoubleElim.get_goal = {response}, status code = {res.status_code}")
        return Position(response["x"], response["y"], -1), res.status_code

    @staticmethod
    def get_items() -> Tuple[List[Dict], int]:
        """Get a list with all current items

        :return: A list will all items currently on the game field. Items are dictionaries that look like: {"id": 0, "x": 0, "y": 0}
        :rtype: Tuple[List[Dict], int]
        """
        res = requests.get(API_URL_GET_ITEMS)
        if res.status_code == 408:
            Logging.get_logger().error(f"DoubleElim.get_items timeout!")
            time.sleep(0.01)
            return DoubleElim.get_items()

        response = json.loads(res.content)
        Logging.get_logger().debug(f"DoubleElim.get_items = {response}, status code = {res.status_code}")
        return response, res.status_code

    @staticmethod
    def get_scores() -> Tuple[Dict, int]:
        """Get the current scores

        :return: A dictionary with all scores included like: {"self":2,"opponent":0}
        :rtype: Tuple[Dict, int]
        """
        res = requests.get(API_URL_GET_SCORES)
        if res.status_code == 408:
            Logging.get_logger().error(f"DoubleElim.get_scores timeout!")
            time.sleep(0.01)
            return DoubleElim.get_scores()

        response = json.loads(res.content)
        Logging.get_logger().debug(f"DoubleElim.get_scores = {response}, status code = {res.status_code}")
        return response, res.status_code
